#! /usr/bin/env python3

import yaml
import datetime
import json
import argparse
import os

parser = argparse.ArgumentParser(
    description='CIS - OSSPI docker scan normalizer')

parser.add_argument('-fn', '--file_name',
                    help='Full name of the scan report file',
                    required=True,
                    type=str)

parser.add_argument('-in', '--image_name',
                    help='Name of the image scanned',
                    required=True,
                    type=str)

parser.add_argument('-s', '--scanner',
                    help='Name of the scanner used',
                    required=False,
                    type=str)

args = vars(parser.parse_args())

scanner = args['scanner'] if args['scanner'] else 'osspi'
file_name = args['file_name']
image_name = args['image_name']
report_path = os.environ.get('REP_PATH_CLI', ".")


def normalize_scan(file_name, scanner):
    scan = {}
    scan['metadata'] = {}
    scan["packages"] = []
    scan["vulnerabilities"] = []
    scan['metadata']['image'] = image_name
    scan['metadata']['scanner'] = scanner
    scan['metadata']['scan_time'] = datetime.datetime.now().isoformat()

    with open(file_name, 'r') as f:
        data = yaml.safe_load(f)

    for k in data.keys():
        package = {}
        package['name'] = data[k]['name']
        package['version'] = data[k].get('version', 'UNKNOWN')
        package['type'] = data[k].get('type', 'UNKNOWN')
        package['path'] = '/'.join(data[k]['detected_paths'].split('#')
                                   [1:3]) if 'detected_paths' in data else 'UNKNOWN'

        if 'license' in data[k] and not data[k]['license'] is None and 'license' not in data[k]['metadata']:
            if 'name' in data[k]['license']:
                package['licenses'] = data[k]['license']['name']
            else:
                package['licenses'] = data[k]['license']
        elif 'license' in data[k]['metadata'] and not data[k]['metadata']['license'] is None:
            if 'name' in data[k]['metadata']['license']:
                package['licenses'] = data[k]['metadata']['license']['name']
            else:
                package['licenses'] = data[k]['metadata']['license']
        else:
            package['licenses'] = 'UNKNOWN'

        if 'vulnerabilities' in data[k] and 'vulnerabilities' not in data[k]['metadata'] and len(data[k]['vulnerabilities']) != 0:
            for vuln in data[k]['vulnerabilities']:
                if 'uid' in vuln and not vuln['uid'] == '':
                    vulnerability = {}
                    vulnerability['id'] = vuln['uid']
                    vulnerability['package_name'] = package['name']
                    vulnerability['package_version'] = package['version']
                    vulnerability['cvss_score'] = vuln.get(
                        'cvss3_basescore', 'UNKNOWN')

                    scan["vulnerabilities"].append(vulnerability)
        elif 'vulnerabilities' in data[k]['metadata'] and len(data[k]['metadata']['vulnerabilities']) != 0:
            for vuln in data[k]['metadata']['vulnerabilities']:
                if 'uid' in vuln and not vuln['uid'] == '':
                    vulnerability = {}
                    vulnerability['id'] = vuln['uid']
                    vulnerability['package_name'] = package['name']
                    vulnerability['package_version'] = package['version']
                    vulnerability['cvss_score'] = vuln.get(
                        'cvss3_basescore', 'UNKNOWN')

                    scan["vulnerabilities"].append(vulnerability)

        scan['packages'].append(package)

    json_data = json.dumps(scan)

    return json_data


json_file = normalize_scan(file_name=file_name, scanner=scanner)
final_file_name = os.path.splitext(file_name)[0] + '.json'
final_file_name = os.path.join(report_path, final_file_name)

with open(final_file_name, 'w') as f:
    f.write(json_file)
